@@ -148,9 +148,9 @@ std::shared_ptr<arrow::Table> parquet___arrow___FileReader__ReadRowGroups2(
 }
 
 // [[parquet::export]]
-int64_t parquet___arrow___FileReader__num_rows(
+r_vec_size parquet___arrow___FileReader__num_rows(
     const std::shared_ptr<parquet::arrow::FileReader>& reader) {
-  return reader->parquet_reader()->metadata()->num_rows();
+  return r_vec_size(reader->parquet_reader()->metadata()->num_rows());
 }
 
 // [[parquet::export]]
@@ -300,9 +300,9 @@ std::shared_ptr<parquet::arrow::FileWriter> parquet___arrow___ParquetFileWriter_
     const std::shared_ptr<arrow::io::OutputStream>& sink,
     const std::shared_ptr<parquet::WriterProperties>& properties,
     const std::shared_ptr<parquet::ArrowWriterProperties>& arrow_properties) {
-  std::unique_ptr<parquet::arrow::FileWriter> writer;
-  PARQUET_THROW_NOT_OK(parquet::arrow::FileWriter::Open(
-      *schema, gc_memory_pool(), sink, properties, arrow_properties, &writer));
+  std::unique_ptr<parquet::arrow::FileWriter> writer =
+      ValueOrStop(parquet::arrow::FileWriter::Open(*schema, gc_memory_pool(), sink,
+                                                   properties, arrow_properties));
   return std::move(writer);
 }
 
