@@ -106,7 +106,7 @@ struct ExecPlanImpl : public ExecPlan {
     if (query_context_.exec_context()->executor() == nullptr) {
       finished_.MarkFinished(Status::Invalid(
           "An exec plan must have an executor for CPU tasks.  To run without threads use "
-          "a SerialExeuctor (the arrow::compute::DeclarationTo... methods should take "
+          "a SerialExecutor (the arrow::compute::DeclarationTo... methods should take "
           "care of this for you and are an easier way to execute an ExecPlan.)"));
       return;
     }
@@ -999,20 +999,30 @@ struct BatchConverter {
   std::shared_ptr<ExecPlan> exec_plan;
 };
 
+// Convert a `Declaration` using `QueryOptions` to a `RecordBatch` generator.
+// Additional outputs:
+// * `out_schema` is the schema for the generated record batches
+// * `out_plan` is the backing `ExecPlan`, which may be stopped to cancel the generation
 Result<AsyncGenerator<std::shared_ptr<RecordBatch>>> DeclarationToRecordBatchGenerator(
     Declaration declaration, QueryOptions options,
-    ::arrow::internal::Executor* cpu_executor, std::shared_ptr<Schema>* out_schema) {
+    ::arrow::internal::Executor* cpu_executor, std::shared_ptr<Schema>* out_schema,
+    std::shared_ptr<ExecPlan>* out_plan) {
+  if (out_plan == nullptr) {
+    std::shared_ptr<ExecPlan> tmp_plan;
+    return DeclarationToRecordBatchGenerator(std::move(declaration), std::move(options),
+                                             cpu_executor, out_schema, &tmp_plan);
+  }
   auto converter = std::make_shared<BatchConverter>();
   ExecContext exec_ctx(options.memory_pool, cpu_executor, options.function_registry);
-  ARROW_ASSIGN_OR_RAISE(std::shared_ptr<ExecPlan> plan,
-                        ExecPlan::Make(options, exec_ctx));
+  std::shared_ptr<ExecPlan>& plan = *out_plan;
+  ARROW_ASSIGN_OR_RAISE(plan, ExecPlan::Make(options, exec_ctx));
   Declaration with_sink = Declaration::Sequence(
       {declaration,
        {"sink", SinkNodeOptions(&converter->exec_batch_gen, &converter->schema)}});
   ARROW_RETURN_NOT_OK(with_sink.AddToPlan(plan.get()));
   ARROW_RETURN_NOT_OK(plan->Validate());
   plan->StartProducing();
-  converter->exec_plan = std::move(plan);
+  converter->exec_plan = plan;
   ARROW_ASSIGN_OR_RAISE(*out_schema, converter->InitializeSchema(options.field_names));
   return [conv = std::move(converter)] { return (*conv)(); };
 }
@@ -1025,6 +1035,7 @@ Result<std::unique_ptr<RecordBatchReader>> DeclarationToReader(Declaration decla
     return Status::Invalid("Cannot use synchronous methods with a custom CPU executor");
   }
   std::shared_ptr<Schema> schema;
+  std::shared_ptr<ExecPlan> plan;
   auto batch_iterator = std::make_unique<Iterator<std::shared_ptr<RecordBatch>>>(
       ::arrow::internal::IterateSynchronously<std::shared_ptr<RecordBatch>>(
           [&](::arrow::internal::Executor* executor)
@@ -1032,14 +1043,16 @@ Result<std::unique_ptr<RecordBatchReader>> DeclarationToReader(Declaration decla
             ExecContext exec_ctx(options.memory_pool, executor,
                                  options.function_registry);
             return DeclarationToRecordBatchGenerator(declaration, std::move(options),
-                                                     executor, &schema);
+                                                     executor, &schema, &plan);
           },
           options.use_threads));
 
   struct PlanReader : RecordBatchReader {
-    PlanReader(std::shared_ptr<Schema> schema,
+    PlanReader(std::shared_ptr<ExecPlan> plan, std::shared_ptr<Schema> schema,
                std::unique_ptr<Iterator<std::shared_ptr<RecordBatch>>> iterator)
-        : schema_(std::move(schema)), iterator_(std::move(iterator)) {}
+        : plan_(std::move(plan)),
+          schema_(std::move(schema)),
+          iterator_(std::move(iterator)) {}
 
     std::shared_ptr<Schema> schema() const override { return schema_; }
 
@@ -1056,19 +1069,26 @@ Result<std::unique_ptr<RecordBatchReader>> DeclarationToReader(Declaration decla
         return Status::OK();
       }
       // End plan and read from generator until finished
+      plan_->StopProducing();
       std::shared_ptr<RecordBatch> batch;
       do {
-        ARROW_RETURN_NOT_OK(ReadNext(&batch));
+        Status st = ReadNext(&batch);
+        if (!st.ok()) {
+          if (st.IsCancelled()) break;  // plan cancelled, so closing is done
+          return st;
+        }
       } while (batch != nullptr);
       iterator_.reset();
       return Status::OK();
     }
 
+    std::shared_ptr<ExecPlan> plan_;
     std::shared_ptr<Schema> schema_;
     std::unique_ptr<Iterator<std::shared_ptr<RecordBatch>>> iterator_;
   };
 
-  return std::make_unique<PlanReader>(std::move(schema), std::move(batch_iterator));
+  return std::make_unique<PlanReader>(std::move(plan), std::move(schema),
+                                      std::move(batch_iterator));
 }
 
 Result<std::unique_ptr<RecordBatchReader>> DeclarationToReader(
