@@ -19,20 +19,36 @@
 
 #include <memory>
 
-#include "arrow/compute/exec/exec_plan.h"
+#include "arrow/acero/exec_plan.h"
 #include "arrow/engine/substrait/visibility.h"
 #include "arrow/type_fwd.h"
 
 namespace arrow {
 namespace engine {
 
-/// Information resulting from converting a Substrait relation.
+/// Execution information resulting from converting a Substrait relation.
 struct ARROW_ENGINE_EXPORT DeclarationInfo {
   /// The compute declaration produced thus far.
-  compute::Declaration declaration;
+  acero::Declaration declaration;
 
   std::shared_ptr<Schema> output_schema;
 };
 
+/// Information resulting from converting a Substrait plan
+struct ARROW_ENGINE_EXPORT PlanInfo {
+  /// The root declaration.
+  ///
+  /// Only plans containing a single top-level relation are supported and so this will
+  /// represent that relation.
+  ///
+  /// This should technically be a RelRoot but some producers use a simple Rel here and so
+  /// Acero currently supports that case.
+  DeclarationInfo root;
+  /// The names of the output fields
+  ///
+  /// If `root` was created from a simple Rel then this will be empty
+  std::vector<std::string> names;
+};
+
 }  // namespace engine
 }  // namespace arrow
