@@ -35,12 +35,6 @@
 
 namespace arrow {
 namespace fs {
-struct GcsCredentialsHolder {
-  // Constructor needed for make_shared
-  explicit GcsCredentialsHolder(std::shared_ptr<google::cloud::Credentials> credentials)
-      : credentials(std::move(credentials)) {}
-  std::shared_ptr<google::cloud::Credentials> credentials;
-};
 
 bool GcsCredentials::Equals(const GcsCredentials& other) const {
   if (holder_->credentials == other.holder_->credentials) {
@@ -58,14 +52,6 @@ namespace gcs = google::cloud::storage;
 using GcsCode = google::cloud::StatusCode;
 using GcsStatus = google::cloud::Status;
 
-// Change the default upload buffer size. In general, sending larger buffers is more
-// efficient with GCS, as each buffer requires a roundtrip to the service. With formatted
-// output (when using `operator<<`), keeping a larger buffer in memory before uploading
-// makes sense.  With unformatted output (the only choice given gcs::io::OutputStream's
-// API) it is better to let the caller provide as large a buffer as they want. The GCS C++
-// client library will upload this buffer with zero copies if possible.
-auto constexpr kUploadBufferSize = 256 * 1024;
-
 struct GcsPath {
   std::string full_path;
   std::string bucket;
@@ -334,40 +320,12 @@ class GcsRandomAccessFile : public arrow::io::RandomAccessFile {
   std::shared_ptr<GcsInputStream> mutable stream_;
 };
 
-google::cloud::Options AsGoogleCloudOptions(const GcsOptions& o) {
-  auto options = google::cloud::Options{};
-  std::string scheme = o.scheme;
-  if (scheme.empty()) scheme = "https";
-  if (scheme == "https") {
-    options.set<google::cloud::UnifiedCredentialsOption>(
-        google::cloud::MakeGoogleDefaultCredentials());
-  } else {
-    options.set<google::cloud::UnifiedCredentialsOption>(
-        google::cloud::MakeInsecureCredentials());
-  }
-  options.set<gcs::UploadBufferSizeOption>(kUploadBufferSize);
-  if (!o.endpoint_override.empty()) {
-    options.set<gcs::RestEndpointOption>(scheme + "://" + o.endpoint_override);
-  }
-  if (o.credentials.holder() && o.credentials.holder()->credentials) {
-    options.set<google::cloud::UnifiedCredentialsOption>(
-        o.credentials.holder()->credentials);
-  }
-  if (o.retry_limit_seconds.has_value()) {
-    options.set<gcs::RetryPolicyOption>(
-        gcs::LimitedTimeRetryPolicy(
-            std::chrono::milliseconds(static_cast<int>(*o.retry_limit_seconds * 1000)))
-            .clone());
-  }
-  return options;
-}
-
 }  // namespace
 
 class GcsFileSystem::Impl {
  public:
   explicit Impl(GcsOptions o)
-      : options_(std::move(o)), client_(AsGoogleCloudOptions(options_)) {}
+      : options_(std::move(o)), client_(internal::AsGoogleCloudOptions(options_)) {}
 
   const GcsOptions& options() const { return options_; }
 
@@ -731,7 +689,7 @@ class GcsFileSystem::Impl {
 };
 
 GcsOptions::GcsOptions() {
-  this->credentials.holder_ = std::make_shared<GcsCredentialsHolder>(
+  this->credentials.holder_ = std::make_shared<internal::GcsCredentialsHolder>(
       google::cloud::MakeGoogleDefaultCredentials());
   this->scheme = "https";
 }
@@ -740,7 +698,8 @@ bool GcsOptions::Equals(const GcsOptions& other) const {
   return credentials.Equals(other.credentials) &&
          endpoint_override == other.endpoint_override && scheme == other.scheme &&
          default_bucket_location == other.default_bucket_location &&
-         retry_limit_seconds == other.retry_limit_seconds;
+         retry_limit_seconds == other.retry_limit_seconds &&
+         project_id == other.project_id;
 }
 
 GcsOptions GcsOptions::Defaults() {
@@ -750,8 +709,8 @@ GcsOptions GcsOptions::Defaults() {
 
 GcsOptions GcsOptions::Anonymous() {
   GcsOptions options{};
-  options.credentials.holder_ =
-      std::make_shared<GcsCredentialsHolder>(google::cloud::MakeInsecureCredentials());
+  options.credentials.holder_ = std::make_shared<internal::GcsCredentialsHolder>(
+      google::cloud::MakeInsecureCredentials());
   options.credentials.anonymous_ = true;
   options.scheme = "http";
   return options;
@@ -760,8 +719,8 @@ GcsOptions GcsOptions::Anonymous() {
 GcsOptions GcsOptions::FromAccessToken(const std::string& access_token,
                                        TimePoint expiration) {
   GcsOptions options{};
-  options.credentials.holder_ =
-      std::make_shared<GcsCredentialsHolder>(google::cloud::MakeAccessTokenCredentials(
+  options.credentials.holder_ = std::make_shared<internal::GcsCredentialsHolder>(
+      google::cloud::MakeAccessTokenCredentials(
           access_token,
           std::chrono::time_point_cast<std::chrono::system_clock::time_point::duration>(
               expiration)));
@@ -775,7 +734,7 @@ GcsOptions GcsOptions::FromImpersonatedServiceAccount(
     const GcsCredentials& base_credentials, const std::string& target_service_account) {
   GcsOptions options{};
   options.credentials = base_credentials;
-  options.credentials.holder_ = std::make_shared<GcsCredentialsHolder>(
+  options.credentials.holder_ = std::make_shared<internal::GcsCredentialsHolder>(
       google::cloud::MakeImpersonateServiceAccountCredentials(
           base_credentials.holder_->credentials, target_service_account));
   options.credentials.target_service_account_ = target_service_account;
@@ -785,7 +744,7 @@ GcsOptions GcsOptions::FromImpersonatedServiceAccount(
 
 GcsOptions GcsOptions::FromServiceAccountCredentials(const std::string& json_object) {
   GcsOptions options{};
-  options.credentials.holder_ = std::make_shared<GcsCredentialsHolder>(
+  options.credentials.holder_ = std::make_shared<internal::GcsCredentialsHolder>(
       google::cloud::MakeServiceAccountCredentials(json_object));
   options.credentials.json_credentials_ = json_object;
   options.scheme = "https";
@@ -844,6 +803,8 @@ Result<GcsOptions> GcsOptions::FromUri(const arrow::internal::Uri& uri,
                                kv.second, "'");
       }
       options.retry_limit_seconds = parsed_seconds;
+    } else if (kv.first == "project_id") {
+      options.project_id = kv.second;
     } else {
       return Status::Invalid("Unexpected query parameter in GCS URI: '", kv.first, "'");
     }
@@ -873,6 +834,12 @@ bool GcsFileSystem::Equals(const FileSystem& other) const {
   return impl_->options().Equals(fs.impl_->options());
 }
 
+Result<std::string> GcsFileSystem::PathFromUri(const std::string& uri_string) const {
+  return internal::PathFromUriHelper(uri_string, {"gs", "gcs"},
+                                     /*accept_local_paths=*/false,
+                                     internal::AuthorityHandlingBehavior::kPrepend);
+}
+
 Result<FileInfo> GcsFileSystem::GetFileInfo(const std::string& path) {
   ARROW_ASSIGN_OR_RAISE(auto p, GcsPath::FromString(path));
   return impl_->GetFileInfo(p);
