@@ -29,6 +29,7 @@
 #include "arrow/ipc/reader.h"
 #include "arrow/status.h"
 #include "arrow/table.h"
+#include "arrow/util/formatting.h"
 #include "arrow/util/string_builder.h"
 #include "arrow/util/uri.h"
 
@@ -275,12 +276,14 @@ Status Ticket::Deserialize(const std::string& serialized, Ticket* out) {
 arrow::Result<FlightInfo> FlightInfo::Make(const Schema& schema,
                                            const FlightDescriptor& descriptor,
                                            const std::vector<FlightEndpoint>& endpoints,
-                                           int64_t total_records, int64_t total_bytes) {
+                                           int64_t total_records, int64_t total_bytes,
+                                           bool ordered) {
   FlightInfo::Data data;
   data.descriptor = descriptor;
   data.endpoints = endpoints;
   data.total_records = total_records;
   data.total_bytes = total_bytes;
+  data.ordered = ordered;
   RETURN_NOT_OK(internal::SchemaToString(schema, &data.schema));
   return FlightInfo(data);
 }
@@ -355,6 +358,7 @@ std::string FlightInfo::ToString() const {
   }
   ss << "] total_records=" << data_.total_records;
   ss << " total_bytes=" << data_.total_bytes;
+  ss << " ordered=" << (data_.ordered ? "true" : "false");
   ss << '>';
   return ss.str();
 }
@@ -364,7 +368,46 @@ bool FlightInfo::Equals(const FlightInfo& other) const {
          data_.descriptor == other.data_.descriptor &&
          data_.endpoints == other.data_.endpoints &&
          data_.total_records == other.data_.total_records &&
-         data_.total_bytes == other.data_.total_bytes;
+         data_.total_bytes == other.data_.total_bytes &&
+         data_.ordered == other.data_.ordered;
+}
+
+std::string CancelFlightInfoRequest::ToString() const {
+  std::stringstream ss;
+  ss << "<CancelFlightInfoRequest info=" << info->ToString() << ">";
+  return ss.str();
+}
+
+bool CancelFlightInfoRequest::Equals(const CancelFlightInfoRequest& other) const {
+  return info == other.info;
+}
+
+arrow::Result<std::string> CancelFlightInfoRequest::SerializeToString() const {
+  pb::CancelFlightInfoRequest pb_request;
+  RETURN_NOT_OK(internal::ToProto(*this, &pb_request));
+
+  std::string out;
+  if (!pb_request.SerializeToString(&out)) {
+    return Status::IOError("Serialized CancelFlightInfoRequest exceeded 2 GiB limit");
+  }
+  return out;
+}
+
+arrow::Result<CancelFlightInfoRequest> CancelFlightInfoRequest::Deserialize(
+    std::string_view serialized) {
+  pb::CancelFlightInfoRequest pb_request;
+  if (serialized.size() > static_cast<size_t>(std::numeric_limits<int>::max())) {
+    return Status::Invalid(
+        "Serialized CancelFlightInfoRequest size should not exceed 2 GiB");
+  }
+  google::protobuf::io::ArrayInputStream input(serialized.data(),
+                                               static_cast<int>(serialized.size()));
+  if (!pb_request.ParseFromZeroCopyStream(&input)) {
+    return Status::Invalid("Not a valid CancelFlightInfoRequest");
+  }
+  CancelFlightInfoRequest out;
+  RETURN_NOT_OK(internal::FromProto(pb_request, &out));
+  return out;
 }
 
 Location::Location() { uri_ = std::make_shared<arrow::internal::Uri>(); }
@@ -444,12 +487,39 @@ std::string FlightEndpoint::ToString() const {
     ss << location.ToString();
     first = false;
   }
-  ss << "]>";
+  ss << "]";
+  auto type = timestamp(TimeUnit::NANO);
+  arrow::internal::StringFormatter<TimestampType> formatter(type.get());
+  ss << " expiration_time=";
+  if (expiration_time) {
+    auto expiration_timestamp = std::chrono::duration_cast<std::chrono::nanoseconds>(
+                                    expiration_time.value().time_since_epoch())
+                                    .count();
+    formatter(expiration_timestamp,
+              [&ss](std::string_view formatted) { ss << formatted; });
+  } else {
+    ss << "null";
+  }
+  ss << ">";
   return ss.str();
 }
 
 bool FlightEndpoint::Equals(const FlightEndpoint& other) const {
-  return ticket == other.ticket && locations == other.locations;
+  if (ticket != other.ticket) {
+    return false;
+  }
+  if (locations != other.locations) {
+    return false;
+  }
+  if (expiration_time.has_value() != other.expiration_time.has_value()) {
+    return false;
+  }
+  if (expiration_time) {
+    if (expiration_time.value() != other.expiration_time.value()) {
+      return false;
+    }
+  }
+  return true;
 }
 
 arrow::Result<std::string> FlightEndpoint::SerializeToString() const {
@@ -478,11 +548,60 @@ arrow::Result<FlightEndpoint> FlightEndpoint::Deserialize(std::string_view seria
   return out;
 }
 
+std::string RenewFlightEndpointRequest::ToString() const {
+  std::stringstream ss;
+  ss << "<RenewFlightEndpointRequest endpoint=" << endpoint.ToString() << ">";
+  return ss.str();
+}
+
+bool RenewFlightEndpointRequest::Equals(const RenewFlightEndpointRequest& other) const {
+  return endpoint == other.endpoint;
+}
+
+arrow::Result<std::string> RenewFlightEndpointRequest::SerializeToString() const {
+  pb::RenewFlightEndpointRequest pb_request;
+  RETURN_NOT_OK(internal::ToProto(*this, &pb_request));
+
+  std::string out;
+  if (!pb_request.SerializeToString(&out)) {
+    return Status::IOError("Serialized RenewFlightEndpointRequest exceeded 2 GiB limit");
+  }
+  return out;
+}
+
+arrow::Result<RenewFlightEndpointRequest> RenewFlightEndpointRequest::Deserialize(
+    std::string_view serialized) {
+  pb::RenewFlightEndpointRequest pb_request;
+  if (serialized.size() > static_cast<size_t>(std::numeric_limits<int>::max())) {
+    return Status::Invalid(
+        "Serialized RenewFlightEndpointRequest size should not exceed 2 GiB");
+  }
+  google::protobuf::io::ArrayInputStream input(serialized.data(),
+                                               static_cast<int>(serialized.size()));
+  if (!pb_request.ParseFromZeroCopyStream(&input)) {
+    return Status::Invalid("Not a valid RenewFlightEndpointRequest");
+  }
+  RenewFlightEndpointRequest out;
+  RETURN_NOT_OK(internal::FromProto(pb_request, &out));
+  return out;
+}
+
 std::string ActionType::ToString() const {
   return arrow::util::StringBuilder("<ActionType type='", type, "' description='",
                                     description, "'>");
 }
 
+const ActionType ActionType::kCancelFlightInfo =
+    ActionType{"CancelFlightInfo",
+               "Explicitly cancel a running FlightInfo.\n"
+               "Request Message: CancelFlightInfoRequest\n"
+               "Response Message: CancelFlightInfoResult"};
+const ActionType ActionType::kRenewFlightEndpoint =
+    ActionType{"RenewFlightEndpoint",
+               "Extend expiration time of the given FlightEndpoint.\n"
+               "Request Message: RenewFlightEndpointRequest\n"
+               "Response Message: Renewed FlightEndpoint"};
+
 bool ActionType::Equals(const ActionType& other) const {
   return type == other.type && description == other.description;
 }
@@ -632,8 +751,73 @@ arrow::Result<Result> Result::Deserialize(std::string_view serialized) {
   return out;
 }
 
+std::string CancelFlightInfoResult::ToString() const {
+  std::stringstream ss;
+  ss << "<CancelFlightInfoResult status=" << status << ">";
+  return ss.str();
+}
+
+bool CancelFlightInfoResult::Equals(const CancelFlightInfoResult& other) const {
+  return status == other.status;
+}
+
+arrow::Result<std::string> CancelFlightInfoResult::SerializeToString() const {
+  pb::CancelFlightInfoResult pb_result;
+  RETURN_NOT_OK(internal::ToProto(*this, &pb_result));
+
+  std::string out;
+  if (!pb_result.SerializeToString(&out)) {
+    return Status::IOError(
+        "Serialized ActionCancelFlightInfoResult exceeded 2 GiB limit");
+  }
+  return out;
+}
+
+arrow::Result<CancelFlightInfoResult> CancelFlightInfoResult::Deserialize(
+    std::string_view serialized) {
+  pb::CancelFlightInfoResult pb_result;
+  if (serialized.size() > static_cast<size_t>(std::numeric_limits<int>::max())) {
+    return Status::Invalid(
+        "Serialized ActionCancelFlightInfoResult size should not exceed 2 GiB");
+  }
+  google::protobuf::io::ArrayInputStream input(serialized.data(),
+                                               static_cast<int>(serialized.size()));
+  if (!pb_result.ParseFromZeroCopyStream(&input)) {
+    return Status::Invalid("Not a valid CancelFlightInfoResult");
+  }
+  CancelFlightInfoResult out;
+  RETURN_NOT_OK(internal::FromProto(pb_result, &out));
+  return out;
+}
+
+std::ostream& operator<<(std::ostream& os, CancelStatus status) {
+  switch (status) {
+    case CancelStatus::kUnspecified:
+      os << "Unspecified";
+      break;
+    case CancelStatus::kCancelled:
+      os << "Cancelled";
+      break;
+    case CancelStatus::kCancelling:
+      os << "Cancelling";
+      break;
+    case CancelStatus::kNotCancellable:
+      os << "NotCancellable";
+      break;
+  }
+  return os;
+}
+
 Status ResultStream::Next(std::unique_ptr<Result>* info) { return Next().Value(info); }
 
+Status ResultStream::Drain() {
+  while (true) {
+    ARROW_ASSIGN_OR_RAISE(auto result, Next());
+    if (!result) break;
+  }
+  return Status::OK();
+}
+
 Status MetadataRecordBatchReader::Next(FlightStreamChunk* next) {
   return Next().Value(next);
 }
