@@ -29,9 +29,10 @@
 
 #include "arrow/util/bit_util.h"
 #include "arrow/util/config.h"  // for ARROW_USE_NATIVE_INT128
+#include "arrow/util/decimal_internal.h"
 #include "arrow/util/endian.h"
 #include "arrow/util/int128_internal.h"
-#include "arrow/util/int_util_internal.h"
+#include "arrow/util/int_util_overflow.h"
 #include "arrow/util/logging.h"
 #include "arrow/util/macros.h"
 
@@ -43,336 +44,15 @@ using internal::SafeSignedAdd;
 using internal::SafeSignedSubtract;
 using internal::SubtractWithOverflow;
 
-static const BasicDecimal128 ScaleMultipliers[] = {
-    BasicDecimal128(1LL),
-    BasicDecimal128(10LL),
-    BasicDecimal128(100LL),
-    BasicDecimal128(1000LL),
-    BasicDecimal128(10000LL),
-    BasicDecimal128(100000LL),
-    BasicDecimal128(1000000LL),
-    BasicDecimal128(10000000LL),
-    BasicDecimal128(100000000LL),
-    BasicDecimal128(1000000000LL),
-    BasicDecimal128(10000000000LL),
-    BasicDecimal128(100000000000LL),
-    BasicDecimal128(1000000000000LL),
-    BasicDecimal128(10000000000000LL),
-    BasicDecimal128(100000000000000LL),
-    BasicDecimal128(1000000000000000LL),
-    BasicDecimal128(10000000000000000LL),
-    BasicDecimal128(100000000000000000LL),
-    BasicDecimal128(1000000000000000000LL),
-    BasicDecimal128(0LL, 10000000000000000000ULL),
-    BasicDecimal128(5LL, 7766279631452241920ULL),
-    BasicDecimal128(54LL, 3875820019684212736ULL),
-    BasicDecimal128(542LL, 1864712049423024128ULL),
-    BasicDecimal128(5421LL, 200376420520689664ULL),
-    BasicDecimal128(54210LL, 2003764205206896640ULL),
-    BasicDecimal128(542101LL, 1590897978359414784ULL),
-    BasicDecimal128(5421010LL, 15908979783594147840ULL),
-    BasicDecimal128(54210108LL, 11515845246265065472ULL),
-    BasicDecimal128(542101086LL, 4477988020393345024ULL),
-    BasicDecimal128(5421010862LL, 7886392056514347008ULL),
-    BasicDecimal128(54210108624LL, 5076944270305263616ULL),
-    BasicDecimal128(542101086242LL, 13875954555633532928ULL),
-    BasicDecimal128(5421010862427LL, 9632337040368467968ULL),
-    BasicDecimal128(54210108624275LL, 4089650035136921600ULL),
-    BasicDecimal128(542101086242752LL, 4003012203950112768ULL),
-    BasicDecimal128(5421010862427522LL, 3136633892082024448ULL),
-    BasicDecimal128(54210108624275221LL, 12919594847110692864ULL),
-    BasicDecimal128(542101086242752217LL, 68739955140067328ULL),
-    BasicDecimal128(5421010862427522170LL, 687399551400673280ULL)};
-
-static const BasicDecimal128 ScaleMultipliersHalf[] = {
-    BasicDecimal128(0ULL),
-    BasicDecimal128(5ULL),
-    BasicDecimal128(50ULL),
-    BasicDecimal128(500ULL),
-    BasicDecimal128(5000ULL),
-    BasicDecimal128(50000ULL),
-    BasicDecimal128(500000ULL),
-    BasicDecimal128(5000000ULL),
-    BasicDecimal128(50000000ULL),
-    BasicDecimal128(500000000ULL),
-    BasicDecimal128(5000000000ULL),
-    BasicDecimal128(50000000000ULL),
-    BasicDecimal128(500000000000ULL),
-    BasicDecimal128(5000000000000ULL),
-    BasicDecimal128(50000000000000ULL),
-    BasicDecimal128(500000000000000ULL),
-    BasicDecimal128(5000000000000000ULL),
-    BasicDecimal128(50000000000000000ULL),
-    BasicDecimal128(500000000000000000ULL),
-    BasicDecimal128(5000000000000000000ULL),
-    BasicDecimal128(2LL, 13106511852580896768ULL),
-    BasicDecimal128(27LL, 1937910009842106368ULL),
-    BasicDecimal128(271LL, 932356024711512064ULL),
-    BasicDecimal128(2710LL, 9323560247115120640ULL),
-    BasicDecimal128(27105LL, 1001882102603448320ULL),
-    BasicDecimal128(271050LL, 10018821026034483200ULL),
-    BasicDecimal128(2710505LL, 7954489891797073920ULL),
-    BasicDecimal128(27105054LL, 5757922623132532736ULL),
-    BasicDecimal128(271050543LL, 2238994010196672512ULL),
-    BasicDecimal128(2710505431LL, 3943196028257173504ULL),
-    BasicDecimal128(27105054312LL, 2538472135152631808ULL),
-    BasicDecimal128(271050543121LL, 6937977277816766464ULL),
-    BasicDecimal128(2710505431213LL, 14039540557039009792ULL),
-    BasicDecimal128(27105054312137LL, 11268197054423236608ULL),
-    BasicDecimal128(271050543121376LL, 2001506101975056384ULL),
-    BasicDecimal128(2710505431213761LL, 1568316946041012224ULL),
-    BasicDecimal128(27105054312137610LL, 15683169460410122240ULL),
-    BasicDecimal128(271050543121376108LL, 9257742014424809472ULL),
-    BasicDecimal128(2710505431213761085LL, 343699775700336640ULL)};
-
-#define BasicDecimal256FromLE(v1, v2, v3, v4) \
-  BasicDecimal256(bit_util::little_endian::ToNative<uint64_t, 4>(v1, v2, v3, v4))
-
-static const BasicDecimal256 ScaleMultipliersDecimal256[] = {
-    BasicDecimal256FromLE({1ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({100ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({100000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({100000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({100000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({100000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({100000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1000000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10000000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({7766279631452241920ULL, 5ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({3875820019684212736ULL, 54ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1864712049423024128ULL, 542ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({200376420520689664ULL, 5421ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({2003764205206896640ULL, 54210ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1590897978359414784ULL, 542101ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({15908979783594147840ULL, 5421010ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({11515845246265065472ULL, 54210108ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({4477988020393345024ULL, 542101086ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({7886392056514347008ULL, 5421010862ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5076944270305263616ULL, 54210108624ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({13875954555633532928ULL, 542101086242ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({9632337040368467968ULL, 5421010862427ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({4089650035136921600ULL, 54210108624275ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({4003012203950112768ULL, 542101086242752ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({3136633892082024448ULL, 5421010862427522ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({12919594847110692864ULL, 54210108624275221ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({68739955140067328ULL, 542101086242752217ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({687399551400673280ULL, 5421010862427522170ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({6873995514006732800ULL, 17316620476856118468ULL, 2ULL, 0ULL}),
-    BasicDecimal256FromLE({13399722918938673152ULL, 7145508105175220139ULL, 29ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {4870020673419870208ULL, 16114848830623546549ULL, 293ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {11806718586779598848ULL, 13574535716559052564ULL, 2938ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {7386721425538678784ULL, 6618148649623664334ULL, 29387ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {80237960548581376ULL, 10841254275107988496ULL, 293873ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {802379605485813760ULL, 16178822382532126880ULL, 2938735ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {8023796054858137600ULL, 14214271235644855872ULL, 29387358ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {6450984253743169536ULL, 13015503840481697412ULL, 293873587ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {9169610316303040512ULL, 1027829888850112811ULL, 2938735877ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {17909126868192198656ULL, 10278298888501128114ULL, 29387358770ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {13070572018536022016ULL, 10549268516463523069ULL, 293873587705ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {1578511669393358848ULL, 13258964796087472617ULL, 2938735877055ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {15785116693933588480ULL, 3462439444907864858ULL, 29387358770557ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {10277214349659471872ULL, 16177650375369096972ULL, 293873587705571ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {10538423128046960640ULL, 14202551164014556797ULL, 2938735877055718ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {13150510911921848320ULL, 12898303124178706663ULL, 29387358770557187ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {2377900603251621888ULL, 18302566799529756941ULL, 293873587705571876ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {5332261958806667264ULL, 17004971331911604867ULL, 2938735877055718769ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {16429131440647569408ULL, 4029016655730084128ULL, 10940614696847636083ULL, 1ULL}),
-    BasicDecimal256FromLE({16717361816799281152ULL, 3396678409881738056ULL,
-                           17172426599928602752ULL, 15ULL}),
-    BasicDecimal256FromLE({1152921504606846976ULL, 15520040025107828953ULL,
-                           5703569335900062977ULL, 159ULL}),
-    BasicDecimal256FromLE({11529215046068469760ULL, 7626447661401876602ULL,
-                           1695461137871974930ULL, 1593ULL}),
-    BasicDecimal256FromLE({4611686018427387904ULL, 2477500319180559562ULL,
-                           16954611378719749304ULL, 15930ULL}),
-    BasicDecimal256FromLE({9223372036854775808ULL, 6328259118096044006ULL,
-                           3525417123811528497ULL, 159309ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 7942358959831785217ULL, 16807427164405733357ULL, 1593091ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 5636613303479645706ULL, 2053574980671369030ULL, 15930919ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 1025900813667802212ULL, 2089005733004138687ULL, 159309191ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 10259008136678022120ULL, 2443313256331835254ULL, 1593091911ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 10356360998232463120ULL, 5986388489608800929ULL, 15930919111ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 11329889613776873120ULL, 4523652674959354447ULL, 159309191113ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 2618431695511421504ULL, 8343038602174441244ULL, 1593091911132ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 7737572881404663424ULL, 9643409726906205977ULL, 15930919111324ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 3588752519208427776ULL, 4200376900514301694ULL, 159309191113245ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 17440781118374726144ULL, 5110280857723913709ULL, 1593091911132452ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 8387114520361296896ULL, 14209320429820033867ULL, 15930919111324522ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 10084168908774762496ULL, 12965995782233477362ULL, 159309191113245227ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 8607968719199866880ULL, 532749306367912313ULL, 1593091911132452277ULL})};
-
-static const BasicDecimal256 ScaleMultipliersHalfDecimal256[] = {
-    BasicDecimal256FromLE({0ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({50ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({500ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({50000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({500000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({50000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({500000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({50000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({500000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({50000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({500000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({50000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({500000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5000000000000000000ULL, 0ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({13106511852580896768ULL, 2ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1937910009842106368ULL, 27ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({932356024711512064ULL, 271ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({9323560247115120640ULL, 2710ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1001882102603448320ULL, 27105ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({10018821026034483200ULL, 271050ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({7954489891797073920ULL, 2710505ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({5757922623132532736ULL, 27105054ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({2238994010196672512ULL, 271050543ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({3943196028257173504ULL, 2710505431ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({2538472135152631808ULL, 27105054312ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({6937977277816766464ULL, 271050543121ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({14039540557039009792ULL, 2710505431213ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({11268197054423236608ULL, 27105054312137ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({2001506101975056384ULL, 271050543121376ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({1568316946041012224ULL, 2710505431213761ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({15683169460410122240ULL, 27105054312137610ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({9257742014424809472ULL, 271050543121376108ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({343699775700336640ULL, 2710505431213761085ULL, 0ULL, 0ULL}),
-    BasicDecimal256FromLE({3436997757003366400ULL, 8658310238428059234ULL, 1ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {15923233496324112384ULL, 12796126089442385877ULL, 14ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {11658382373564710912ULL, 17280796452166549082ULL, 146ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {5903359293389799424ULL, 6787267858279526282ULL, 1469ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {3693360712769339392ULL, 12532446361666607975ULL, 14693ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {40118980274290688ULL, 14643999174408770056ULL, 146936ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {401189802742906880ULL, 17312783228120839248ULL, 1469367ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {4011898027429068800ULL, 7107135617822427936ULL, 14693679ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {3225492126871584768ULL, 15731123957095624514ULL, 146936793ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {13808177195006296064ULL, 9737286981279832213ULL, 1469367938ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {8954563434096099328ULL, 5139149444250564057ULL, 14693679385ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {15758658046122786816ULL, 14498006295086537342ULL, 146936793852ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {10012627871551455232ULL, 15852854434898512116ULL, 1469367938527ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {7892558346966794240ULL, 10954591759308708237ULL, 14693679385278ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {5138607174829735936ULL, 17312197224539324294ULL, 146936793852785ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {14492583600878256128ULL, 7101275582007278398ULL, 1469367938527859ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {15798627492815699968ULL, 15672523598944129139ULL, 14693679385278593ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {10412322338480586752ULL, 9151283399764878470ULL, 146936793852785938ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {11889503016258109440ULL, 17725857702810578241ULL, 1469367938527859384ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {8214565720323784704ULL, 11237880364719817872ULL, 14693679385278593849ULL, 0ULL}),
-    BasicDecimal256FromLE(
-        {8358680908399640576ULL, 1698339204940869028ULL, 17809585336819077184ULL, 7ULL}),
-    BasicDecimal256FromLE({9799832789158199296ULL, 16983392049408690284ULL,
-                           12075156704804807296ULL, 79ULL}),
-    BasicDecimal256FromLE({5764607523034234880ULL, 3813223830700938301ULL,
-                           10071102605790763273ULL, 796ULL}),
-    BasicDecimal256FromLE({2305843009213693952ULL, 1238750159590279781ULL,
-                           8477305689359874652ULL, 7965ULL}),
-    BasicDecimal256FromLE({4611686018427387904ULL, 12387501595902797811ULL,
-                           10986080598760540056ULL, 79654ULL}),
-    BasicDecimal256FromLE({9223372036854775808ULL, 13194551516770668416ULL,
-                           17627085619057642486ULL, 796545ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 2818306651739822853ULL, 10250159527190460323ULL, 7965459ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 9736322443688676914ULL, 10267874903356845151ULL, 79654595ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 5129504068339011060ULL, 10445028665020693435ULL, 796545955ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 14401552535971007368ULL, 12216566281659176272ULL, 7965459555ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 14888316843743212368ULL, 11485198374334453031ULL, 79654595556ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 1309215847755710752ULL, 4171519301087220622ULL, 796545955566ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 13092158477557107520ULL, 4821704863453102988ULL, 7965459555662ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 1794376259604213888ULL, 11323560487111926655ULL, 79654595556622ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 17943762596042138880ULL, 2555140428861956854ULL, 796545955566226ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 13416929297035424256ULL, 7104660214910016933ULL, 7965459555662261ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 5042084454387381248ULL, 15706369927971514489ULL, 79654595556622613ULL}),
-    BasicDecimal256FromLE(
-        {0ULL, 13527356396454709248ULL, 9489746690038731964ULL, 796545955566226138ULL})};
-
-#undef BasicDecimal256FromLE
-
 #ifdef ARROW_USE_NATIVE_INT128
 static constexpr uint64_t kInt64Mask = 0xFFFFFFFFFFFFFFFF;
 #else
 static constexpr uint64_t kInt32Mask = 0xFFFFFFFF;
 #endif
 
-// same as ScaleMultipliers[38] - 1
-static constexpr BasicDecimal128 kMaxValue =
-    BasicDecimal128(5421010862427522170LL, 687399551400673280ULL - 1);
-
-constexpr int BasicDecimal128::kMaxPrecision;
-constexpr int BasicDecimal128::kMaxScale;
+// same as kDecimal128PowersOfTen[38] - 1
+static constexpr BasicDecimal128 kMaxDecimal128Value{5421010862427522170LL,
+                                                     687399551400673280ULL - 1};
 
 BasicDecimal128& BasicDecimal128::Negate() {
   uint64_t result_lo = ~low_bits() + 1;
@@ -394,7 +74,7 @@ BasicDecimal128 BasicDecimal128::Abs(const BasicDecimal128& in) {
 bool BasicDecimal128::FitsInPrecision(int32_t precision) const {
   DCHECK_GT(precision, 0);
   DCHECK_LE(precision, 38);
-  return BasicDecimal128::Abs(*this) < ScaleMultipliers[precision];
+  return BasicDecimal128::Abs(*this) < kDecimal128PowersOfTen[precision];
 }
 
 BasicDecimal128& BasicDecimal128::operator+=(const BasicDecimal128& right) {
@@ -799,7 +479,7 @@ static DecimalStatus BuildFromArray(BasicDecimal256* value, const uint32_t* arra
   if (status != DecimalStatus::kSuccess) {
     return status;
   }
-  *value = result_array;
+  *value = BasicDecimal256(result_array);
   return DecimalStatus::kSuccess;
 }
 
@@ -951,14 +631,6 @@ DecimalStatus BasicDecimal128::Divide(const BasicDecimal128& divisor,
   return DecimalDivide(*this, divisor, result, remainder);
 }
 
-bool operator==(const BasicDecimal128& left, const BasicDecimal128& right) {
-  return left.high_bits() == right.high_bits() && left.low_bits() == right.low_bits();
-}
-
-bool operator!=(const BasicDecimal128& left, const BasicDecimal128& right) {
-  return !operator==(left, right);
-}
-
 bool operator<(const BasicDecimal128& left, const BasicDecimal128& right) {
   return left.high_bits() < right.high_bits() ||
          (left.high_bits() == right.high_bits() && left.low_bits() < right.low_bits());
@@ -1072,7 +744,7 @@ void BasicDecimal128::GetWholeAndFraction(int scale, BasicDecimal128* whole,
   DCHECK_GE(scale, 0);
   DCHECK_LE(scale, 38);
 
-  BasicDecimal128 multiplier(ScaleMultipliers[scale]);
+  BasicDecimal128 multiplier(kDecimal128PowersOfTen[scale]);
   auto s = Divide(multiplier, whole, fraction);
   DCHECK_EQ(s, DecimalStatus::kSuccess);
 }
@@ -1081,29 +753,29 @@ const BasicDecimal128& BasicDecimal128::GetScaleMultiplier(int32_t scale) {
   DCHECK_GE(scale, 0);
   DCHECK_LE(scale, 38);
 
-  return ScaleMultipliers[scale];
+  return kDecimal128PowersOfTen[scale];
 }
 
 const BasicDecimal128& BasicDecimal128::GetHalfScaleMultiplier(int32_t scale) {
   DCHECK_GE(scale, 0);
   DCHECK_LE(scale, 38);
 
-  return ScaleMultipliersHalf[scale];
+  return kDecimal128HalfPowersOfTen[scale];
 }
 
-const BasicDecimal128& BasicDecimal128::GetMaxValue() { return kMaxValue; }
+const BasicDecimal128& BasicDecimal128::GetMaxValue() { return kMaxDecimal128Value; }
 
 BasicDecimal128 BasicDecimal128::GetMaxValue(int32_t precision) {
   DCHECK_GE(precision, 0);
   DCHECK_LE(precision, 38);
-  return ScaleMultipliers[precision] - 1;
+  return kDecimal128PowersOfTen[precision] - 1;
 }
 
 BasicDecimal128 BasicDecimal128::IncreaseScaleBy(int32_t increase_by) const {
   DCHECK_GE(increase_by, 0);
   DCHECK_LE(increase_by, 38);
 
-  return (*this) * ScaleMultipliers[increase_by];
+  return (*this) * kDecimal128PowersOfTen[increase_by];
 }
 
 BasicDecimal128 BasicDecimal128::ReduceScaleBy(int32_t reduce_by, bool round) const {
@@ -1114,13 +786,13 @@ BasicDecimal128 BasicDecimal128::ReduceScaleBy(int32_t reduce_by, bool round) co
     return *this;
   }
 
-  BasicDecimal128 divisor(ScaleMultipliers[reduce_by]);
+  BasicDecimal128 divisor(kDecimal128PowersOfTen[reduce_by]);
   BasicDecimal128 result;
   BasicDecimal128 remainder;
   auto s = Divide(divisor, &result, &remainder);
   DCHECK_EQ(s, DecimalStatus::kSuccess);
   if (round) {
-    auto divisor_half = ScaleMultipliersHalf[reduce_by];
+    auto divisor_half = kDecimal128HalfPowersOfTen[reduce_by];
     if (remainder.Abs() >= divisor_half) {
       result += Sign();
     }
@@ -1138,9 +810,6 @@ int32_t BasicDecimal128::CountLeadingBinaryZeros() const {
   }
 }
 
-constexpr int BasicDecimal256::kMaxPrecision;
-constexpr int BasicDecimal256::kMaxScale;
-
 BasicDecimal256& BasicDecimal256::Negate() {
   auto array_le = bit_util::little_endian::Make(&array_);
   uint64_t carry = 1;
@@ -1188,14 +857,14 @@ BasicDecimal256& BasicDecimal256::operator<<=(uint32_t bits) {
   if (bits == 0) {
     return *this;
   }
-  int cross_word_shift = bits / 64;
-  if (static_cast<size_t>(cross_word_shift) >= array_.size()) {
+  const int cross_word_shift = bits / 64;
+  if (cross_word_shift >= kNumWords) {
     array_ = {0, 0, 0, 0};
     return *this;
   }
   uint32_t in_word_shift = bits % 64;
   auto array_le = bit_util::little_endian::Make(&array_);
-  for (int i = static_cast<int>(array_.size() - 1); i >= cross_word_shift; i--) {
+  for (int i = kNumWords - 1; i >= cross_word_shift; i--) {
     // Account for shifts larger then 64 bits
     array_le[i] = array_le[i - cross_word_shift];
     array_le[i] <<= in_word_shift;
@@ -1209,6 +878,35 @@ BasicDecimal256& BasicDecimal256::operator<<=(uint32_t bits) {
   return *this;
 }
 
+BasicDecimal256& BasicDecimal256::operator>>=(uint32_t bits) {
+  if (bits == 0) {
+    return *this;
+  }
+  const uint64_t extended =
+      static_cast<uint64_t>(static_cast<int64_t>(array_[kHighWordIndex]) >> 63);
+  const int cross_word_shift = bits / 64;
+  if (cross_word_shift >= kNumWords) {
+    array_.fill(extended);
+    return *this;
+  }
+  const uint32_t in_word_shift = bits % 64;
+  const auto array_le = little_endian_array();
+  // Initialize with sign-extended words
+  WordArray shifted_le;
+  shifted_le.fill(extended);
+  // Iterate from LSW to MSW
+  for (int i = cross_word_shift; i < kNumWords; ++i) {
+    shifted_le[i - cross_word_shift] = array_le[i] >> in_word_shift;
+    if (in_word_shift != 0) {
+      const uint64_t carry_bits = (i + 1 < kNumWords ? array_le[i + 1] : extended)
+                                  << (64 - in_word_shift);
+      shifted_le[i - cross_word_shift] |= carry_bits;
+    }
+  }
+  array_ = bit_util::little_endian::ToNative(shifted_le);
+  return *this;
+}
+
 BasicDecimal256& BasicDecimal256::operator*=(const BasicDecimal256& right) {
   // Since the max value of BasicDecimal256 is supposed to be 1e76 - 1 and the
   // min the negation taking the absolute values here should always be safe.
@@ -1240,7 +938,7 @@ BasicDecimal256 BasicDecimal256::IncreaseScaleBy(int32_t increase_by) const {
   DCHECK_GE(increase_by, 0);
   DCHECK_LE(increase_by, 76);
 
-  return (*this) * ScaleMultipliersDecimal256[increase_by];
+  return (*this) * kDecimal256PowersOfTen[increase_by];
 }
 
 BasicDecimal256 BasicDecimal256::ReduceScaleBy(int32_t reduce_by, bool round) const {
@@ -1251,13 +949,13 @@ BasicDecimal256 BasicDecimal256::ReduceScaleBy(int32_t reduce_by, bool round) co
     return *this;
   }
 
-  BasicDecimal256 divisor(ScaleMultipliersDecimal256[reduce_by]);
+  BasicDecimal256 divisor(kDecimal256PowersOfTen[reduce_by]);
   BasicDecimal256 result;
   BasicDecimal256 remainder;
   auto s = Divide(divisor, &result, &remainder);
   DCHECK_EQ(s, DecimalStatus::kSuccess);
   if (round) {
-    auto divisor_half = ScaleMultipliersHalfDecimal256[reduce_by];
+    auto divisor_half = kDecimal256HalfPowersOfTen[reduce_by];
     if (remainder.Abs() >= divisor_half) {
       result += Sign();
     }
@@ -1268,27 +966,27 @@ BasicDecimal256 BasicDecimal256::ReduceScaleBy(int32_t reduce_by, bool round) co
 bool BasicDecimal256::FitsInPrecision(int32_t precision) const {
   DCHECK_GT(precision, 0);
   DCHECK_LE(precision, 76);
-  return BasicDecimal256::Abs(*this) < ScaleMultipliersDecimal256[precision];
+  return BasicDecimal256::Abs(*this) < kDecimal256PowersOfTen[precision];
 }
 
 const BasicDecimal256& BasicDecimal256::GetScaleMultiplier(int32_t scale) {
   DCHECK_GE(scale, 0);
   DCHECK_LE(scale, 76);
 
-  return ScaleMultipliersDecimal256[scale];
+  return kDecimal256PowersOfTen[scale];
 }
 
 const BasicDecimal256& BasicDecimal256::GetHalfScaleMultiplier(int32_t scale) {
   DCHECK_GE(scale, 0);
   DCHECK_LE(scale, 76);
 
-  return ScaleMultipliersHalfDecimal256[scale];
+  return kDecimal256HalfPowersOfTen[scale];
 }
 
 BasicDecimal256 BasicDecimal256::GetMaxValue(int32_t precision) {
   DCHECK_GE(precision, 0);
   DCHECK_LE(precision, 76);
-  return ScaleMultipliersDecimal256[precision] + (-1);
+  return kDecimal256PowersOfTen[precision] + (-1);
 }
 
 BasicDecimal256 operator*(const BasicDecimal256& left, const BasicDecimal256& right) {
